@@ -73,7 +73,9 @@ use super::ffi::cl_ext::{
     cl_device_pci_bus_info_khr,
     CL_DEVICE_INTEGER_DOT_PRODUCT_ACCELERATION_PROPERTIES_4x8BIT_PACKED_KHR,
     CL_DEVICE_AVAILABLE_ASYNC_QUEUES_AMD, CL_DEVICE_BOARD_NAME_AMD,
-    CL_DEVICE_COMPUTE_CAPABILITY_MAJOR_NV, CL_DEVICE_COMPUTE_CAPABILITY_MINOR_NV,
+    CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR,
+    CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR, CL_DEVICE_COMPUTE_CAPABILITY_MAJOR_NV,
+    CL_DEVICE_COMPUTE_CAPABILITY_MINOR_NV, CL_DEVICE_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR,
     CL_DEVICE_FEATURE_CAPABILITIES_INTEL, CL_DEVICE_GFXIP_MAJOR_AMD, CL_DEVICE_GFXIP_MINOR_AMD,
     CL_DEVICE_GLOBAL_FREE_MEMORY_AMD, CL_DEVICE_GLOBAL_MEM_CHANNELS_AMD,
     CL_DEVICE_GLOBAL_MEM_CHANNEL_BANKS_AMD, CL_DEVICE_GLOBAL_MEM_CHANNEL_BANK_WIDTH_AMD,
@@ -88,11 +90,12 @@ use super::ffi::cl_ext::{
     CL_DEVICE_PCIE_ID_AMD, CL_DEVICE_PCI_BUS_ID_NV, CL_DEVICE_PCI_BUS_INFO_KHR,
     CL_DEVICE_PCI_SLOT_ID_NV, CL_DEVICE_PREFERRED_CONSTANT_BUFFER_SIZE_AMD,
     CL_DEVICE_PREFERRED_WORK_GROUP_SIZE_AMD, CL_DEVICE_PROFILING_TIMER_OFFSET_AMD,
-    CL_DEVICE_REGISTERS_PER_BLOCK_NV, CL_DEVICE_SIMD_INSTRUCTION_WIDTH_AMD,
-    CL_DEVICE_SIMD_PER_COMPUTE_UNIT_AMD, CL_DEVICE_SIMD_WIDTH_AMD,
-    CL_DEVICE_THREAD_TRACE_SUPPORTED_AMD, CL_DEVICE_TOPOLOGY_AMD, CL_DEVICE_UUID_KHR,
-    CL_DEVICE_WARP_SIZE_NV, CL_DEVICE_WAVEFRONT_WIDTH_AMD, CL_DRIVER_UUID_KHR, CL_LUID_SIZE_KHR,
-    CL_UUID_SIZE_KHR,
+    CL_DEVICE_REGISTERS_PER_BLOCK_NV, CL_DEVICE_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR,
+    CL_DEVICE_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR, CL_DEVICE_SEMAPHORE_TYPES_KHR,
+    CL_DEVICE_SIMD_INSTRUCTION_WIDTH_AMD, CL_DEVICE_SIMD_PER_COMPUTE_UNIT_AMD,
+    CL_DEVICE_SIMD_WIDTH_AMD, CL_DEVICE_THREAD_TRACE_SUPPORTED_AMD, CL_DEVICE_TOPOLOGY_AMD,
+    CL_DEVICE_UUID_KHR, CL_DEVICE_WARP_SIZE_NV, CL_DEVICE_WAVEFRONT_WIDTH_AMD, CL_DRIVER_UUID_KHR,
+    CL_LUID_SIZE_KHR, CL_UUID_SIZE_KHR,
 };
 use super::info_type::InfoType;
 #[allow(unused_imports)]
@@ -419,6 +422,8 @@ pub fn get_device_info(
         | CL_DEVICE_DEVICE_ENQUEUE_CAPABILITIES // CL_VERSION_3_0
         | CL_DEVICE_INTEGER_DOT_PRODUCT_CAPABILITIES_KHR
         | CL_DEVICE_FEATURE_CAPABILITIES_INTEL // cl_intel_device_attribute_query
+        | CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR // cl_khr_command_buffer
+        | CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR // cl_khr_command_buffer
         => {
             api_info_value!(get_value, cl_ulong, clGetDeviceInfo);
             Ok(InfoType::Ulong(get_value(device, param_name)?))
@@ -455,6 +460,16 @@ pub fn get_device_info(
             Ok(InfoType::Ptr(get_value(device, param_name)?))
         }
 
+        CL_DEVICE_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR // cl_khr_external_memory
+        | CL_DEVICE_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR // cl_khr_external_semaphore
+        | CL_DEVICE_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR // cl_khr_external_semaphore
+        | CL_DEVICE_SEMAPHORE_TYPES_KHR // cl_khr_semaphore
+        => {
+            api_info_vector!(get_vec, cl_uint, clGetDeviceInfo);
+            let size = get_size(device, param_name)?;
+            Ok(InfoType::VecUshort(get_vec(device, param_name, size)?))
+        }
+
         CL_DEVICE_PARTITION_AFFINITY_DOMAIN => {
             api_info_vector!(get_vec, cl_ulong, clGetDeviceInfo);
             let size = get_size(device, param_name)?;
@@ -1679,6 +1694,75 @@ mod tests {
             ),
         };
 
+        match get_device_info(device_id, CL_DEVICE_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR) {
+            Ok(value) => {
+                let value: Vec<u32> = value.into();
+                println!(
+                    "CL_DEVICE_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR: {:?}",
+                    value
+                )
+            }
+            Err(e) => println!(
+                "OpenCL error, CL_DEVICE_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR: {}",
+                ClError(e)
+            ),
+        };
+
+        match get_device_info(device_id, CL_DEVICE_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR) {
+            Ok(value) => {
+                let value: Vec<u32> = value.into();
+                println!("CL_DEVICE_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR: {:?}", value)
+            }
+            Err(e) => println!(
+                "OpenCL error, CL_DEVICE_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR: {}",
+                ClError(e)
+            ),
+        };
+
+        match get_device_info(device_id, CL_DEVICE_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR) {
+            Ok(value) => {
+                let value: Vec<u32> = value.into();
+                println!("CL_DEVICE_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR: {:?}", value)
+            }
+            Err(e) => println!(
+                "OpenCL error, CL_DEVICE_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR: {}",
+                ClError(e)
+            ),
+        };
+
+        match get_device_info(device_id, CL_DEVICE_SEMAPHORE_TYPES_KHR) {
+            Ok(value) => {
+                let value: Vec<u32> = value.into();
+                println!("CL_DEVICE_SEMAPHORE_TYPES_KHR: {:?}", value)
+            }
+            Err(e) => println!(
+                "OpenCL error, CL_DEVICE_SEMAPHORE_TYPES_KHR: {}",
+                ClError(e)
+            ),
+        };
+
+        match get_device_info(device_id, CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR) {
+            Ok(value) => {
+                let value: cl_ulong = value.into();
+                println!("CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR: {}", value)
+            }
+            Err(e) => println!(
+                "OpenCL error, CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR: {}",
+                ClError(e)
+            ),
+        };
+
+        match get_device_info(device_id, CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR) {
+            Ok(value) => {
+                let value: cl_ulong = value.into();
+                println!("CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR: {}", value)
+            }
+            Err(e) => println!(
+                "OpenCL error, CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR: {}",
+                ClError(e)
+            ),
+        };
+
         // CL_VERSION_2_0
         if is_opencl_2 {
             let value = get_device_info(device_id, CL_DEVICE_IMAGE_PITCH_ALIGNMENT).unwrap();
