@@ -29,6 +29,54 @@ pub use cl_sys::{
 
 use libc::{c_void, intptr_t, size_t};
 
+// cl_khr_command_buffer
+
+pub type cl_device_command_buffer_capabilities_khr = cl_bitfield;
+pub type cl_command_buffer_khr = *mut c_void;
+pub type cl_sync_point_khr = cl_uint;
+pub type cl_command_buffer_info_khr = cl_uint;
+pub type cl_command_buffer_state_khr = cl_uint;
+pub type cl_command_buffer_properties_khr = cl_properties;
+pub type cl_command_buffer_flags_khr = cl_bitfield;
+pub type cl_ndrange_kernel_command_properties_khr = cl_properties;
+pub type cl_mutable_command_khr = *mut c_void;
+
+pub const CL_DEVICE_COMMAND_BUFFER_CAPABILITIES_KHR: cl_device_info = 0x12A9;
+pub const CL_DEVICE_COMMAND_BUFFER_REQUIRED_QUEUE_PROPERTIES_KHR: cl_device_info = 0x12AA;
+
+pub const CL_COMMAND_BUFFER_CAPABILITY_KERNEL_PRINTF_KHR:
+    cl_device_command_buffer_capabilities_khr = 1 << 0;
+pub const CL_COMMAND_BUFFER_CAPABILITY_DEVICE_SIDE_ENQUEUE_KHR:
+    cl_device_command_buffer_capabilities_khr = 1 << 1;
+pub const CL_COMMAND_BUFFER_CAPABILITY_SIMULTANEOUS_USE_KHR:
+    cl_device_command_buffer_capabilities_khr = 1 << 2;
+pub const CL_COMMAND_BUFFER_CAPABILITY_OUT_OF_ORDER_KHR: cl_device_command_buffer_capabilities_khr =
+    1 << 3;
+
+pub const CL_COMMAND_BUFFER_FLAGS_KHR: cl_command_buffer_properties_khr = 0x1293;
+
+pub const CL_COMMAND_BUFFER_SIMULTANEOUS_USE_KHR: cl_command_buffer_flags_khr = 1 << 0;
+
+// error codes
+pub const CL_INVALID_COMMAND_BUFFER_KHR: cl_int = -1138;
+pub const CL_INVALID_SYNC_POINT_WAIT_LIST_KHR: cl_int = -1139;
+pub const CL_INCOMPATIBLE_COMMAND_QUEUE_KHR: cl_int = -1140;
+
+pub const CL_COMMAND_BUFFER_QUEUES_KHR: cl_command_buffer_info_khr = 0x1294;
+pub const CL_COMMAND_BUFFER_NUM_QUEUES_KHR: cl_command_buffer_info_khr = 0x1295;
+pub const CL_COMMAND_BUFFER_REFERENCE_COUNT_KHR: cl_command_buffer_info_khr = 0x1296;
+pub const CL_COMMAND_BUFFER_STATE_KHR: cl_command_buffer_info_khr = 0x1297;
+pub const CL_COMMAND_BUFFER_PROPERTIES_ARRAY_KHR: cl_command_buffer_info_khr = 0x1298;
+
+pub const CL_COMMAND_BUFFER_STATE_RECORDING_KHR: cl_command_buffer_state_khr = 0;
+pub const CL_COMMAND_BUFFER_STATE_EXECUTABLE_KHR: cl_command_buffer_state_khr = 1;
+pub const CL_COMMAND_BUFFER_STATE_PENDING_KHR: cl_command_buffer_state_khr = 2;
+pub const CL_COMMAND_BUFFER_STATE_INVALID_KHR: cl_command_buffer_state_khr = 3;
+
+pub const CL_COMMAND_COMMAND_BUFFER_KHR: cl_command_type = 0x12A8;
+
+// cl_khr_fp64
+
 // pub const CL_DEVICE_DOUBLE_FP_CONFIG: cl_device_info = 0x1032;
 // pub const CL_DEVICE_HALF_FP_CONFIG: cl_device_info = 0x1033;
 
@@ -350,11 +398,95 @@ pub const CL_DEVICE_INTEGER_DOT_PRODUCT_ACCELERATION_PROPERTIES_8BIT_KHR: cl_dev
 pub const CL_DEVICE_INTEGER_DOT_PRODUCT_ACCELERATION_PROPERTIES_4x8BIT_PACKED_KHR: cl_device_info =
     0x1075;
 
+// cl_khr_external_memory
+
+pub type cl_external_memory_handle_type_khr = cl_uint;
+
+pub const CL_PLATFORM_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR: cl_platform_info = 0x2044;
+
+pub const CL_DEVICE_EXTERNAL_MEMORY_IMPORT_HANDLE_TYPES_KHR: cl_device_info = 0x204F;
+
+// cl_mem_properties
+pub const CL_DEVICE_HANDLE_LIST_KHR: cl_ulong = 0x2051;
+pub const CL_DEVICE_HANDLE_LIST_END_KHR: cl_ulong = 0;
+
+pub const CL_COMMAND_ACQUIRE_EXTERNAL_MEM_OBJECTS_KHR: cl_command_type = 0x2047;
+pub const CL_COMMAND_RELEASE_EXTERNAL_MEM_OBJECTS_KHR: cl_command_type = 0x2048;
+
+// cl_khr_external_memory_dma_buf
+pub const CL_EXTERNAL_MEMORY_HANDLE_DMA_BUF_KHR: cl_external_memory_handle_type_khr = 0x2067;
+
+// cl_khr_external_memory_dx
+pub const CL_EXTERNAL_MEMORY_HANDLE_D3D11_TEXTURE_KHR: cl_external_memory_handle_type_khr = 0x2063;
+pub const CL_EXTERNAL_MEMORY_HANDLE_D3D11_TEXTURE_KMT_KHR: cl_external_memory_handle_type_khr =
+    0x2064;
+pub const CL_EXTERNAL_MEMORY_HANDLE_D3D12_HEAP_KHR: cl_external_memory_handle_type_khr = 0x2065;
+pub const CL_EXTERNAL_MEMORY_HANDLE_D3D12_RESOURCE_KHR: cl_external_memory_handle_type_khr = 0x2066;
+
+// cl_khr_external_memory_opaque_fd
+pub const CL_EXTERNAL_MEMORY_HANDLE_OPAQUE_FD_KHR: cl_external_memory_handle_type_khr = 0x2060;
+
+// cl_khr_external_memory_win32
+pub const CL_EXTERNAL_MEMORY_HANDLE_OPAQUE_WIN32_KHR: cl_external_memory_handle_type_khr = 0x2061;
+pub const CL_EXTERNAL_MEMORY_HANDLE_OPAQUE_WIN32_KMT_KHR: cl_external_memory_handle_type_khr =
+    0x2062;
+
+// cl_khr_external_semaphore
+pub type cl_semaphore_khr = *mut c_void;
+pub type cl_external_semaphore_handle_type_khr = cl_uint;
+pub type cl_semaphore_properties_khr = cl_properties;
+
+pub const CL_PLATFORM_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR: cl_platform_info = 0x2037;
+pub const CL_PLATFORM_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR: cl_platform_info = 0x2038;
+
+pub const CL_DEVICE_SEMAPHORE_IMPORT_HANDLE_TYPES_KHR: cl_device_info = 0x204D;
+pub const CL_DEVICE_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR: cl_device_info = 0x204E;
+
+pub const CL_SEMAPHORE_EXPORT_HANDLE_TYPES_KHR: cl_semaphore_properties_khr = 0x203F;
+pub const CL_SEMAPHORE_EXPORT_HANDLE_TYPES_LIST_END_KHR: cl_semaphore_properties_khr = 0;
+
+// cl_khr_external_semaphore_dx_fence
+pub const CL_SEMAPHORE_HANDLE_D3D12_FENCE_KHR: cl_external_semaphore_handle_type_khr = 0x2059;
+
+// cl_khr_external_semaphore_opaque_fd
+pub const CL_SEMAPHORE_HANDLE_OPAQUE_FD_KHR: cl_external_semaphore_handle_type_khr = 0x2055;
+
+// cl_khr_external_semaphore_sync_fd
+pub const CL_SEMAPHORE_HANDLE_SYNC_FD_KHR: cl_external_semaphore_handle_type_khr = 0x2058;
+
+// cl_khr_external_semaphore_win32
+pub const CL_SEMAPHORE_HANDLE_OPAQUE_WIN32_KHR: cl_external_semaphore_handle_type_khr = 0x2056;
+pub const CL_SEMAPHORE_HANDLE_OPAQUE_WIN32_KMT_KHR: cl_external_semaphore_handle_type_khr = 0x2057;
+
+// cl_khr_semaphore
+pub type cl_semaphore_info_khr = cl_uint;
+pub type cl_semaphore_type_khr = cl_uint;
+pub type cl_semaphore_payload_khr = cl_ulong;
+
+pub const CL_SEMAPHORE_TYPE_BINARY_KHR: cl_semaphore_type_khr = 1;
+
+pub const CL_PLATFORM_SEMAPHORE_TYPES_KHR: cl_platform_info = 0x2036;
+
+pub const CL_DEVICE_SEMAPHORE_TYPES_KHR: cl_device_info = 0x204C;
+
+pub const CL_SEMAPHORE_CONTEXT_KHR: cl_semaphore_info_khr = 0x2039;
+pub const CL_SEMAPHORE_REFERENCE_COUNT_KHR: cl_semaphore_info_khr = 0x203A;
+pub const CL_SEMAPHORE_PROPERTIES_KHR: cl_semaphore_info_khr = 0x203B;
+pub const CL_SEMAPHORE_PAYLOAD_KHR: cl_semaphore_info_khr = 0x203C;
+
+pub const CL_SEMAPHORE_TYPE_KHR: cl_semaphore_info_khr = 0x203D;
+
+pub const CL_COMMAND_SEMAPHORE_WAIT_KHR: cl_command_type = 0x2042;
+pub const CL_COMMAND_SEMAPHORE_SIGNAL_KHR: cl_command_type = 0x2043;
+
+// Error codes
+pub const CL_INVALID_SEMAPHORE_KHR: cl_int = -1142;
+
 // cl_arm_import_memory extension
 
 pub type cl_import_properties_arm = intptr_t;
 
-/// Default and valid proporties name for cl_arm_import_memory
+/// Default and valid properties name for cl_arm_import_memory
 pub const CL_IMPORT_TYPE_ARM: cl_import_properties_arm = 0x40B2;
 
 /// Host process memory type default value for CL_IMPORT_TYPE_ARM property
@@ -378,8 +510,9 @@ pub const CL_IMPORT_ANDROID_HARDWARE_BUFFER_PLANE_INDEX_ARM: cl_import_propertie
 /// Index of layer in a multilayer hardware buffer
 pub const CL_IMPORT_ANDROID_HARDWARE_BUFFER_LAYER_INDEX_ARM: cl_import_properties_arm = 0x41F0;
 
-// Import memory size value to indicate a size for the whole buffer.
-// pub const CL_IMPORT_ANDROID_HARDWARE_BUFFER_LAYER_INDEX_ARM: usize = SIZE_MAX;
+/// Import memory size value to indicate a size for the whole buffer.
+pub const CL_IMPORT_MEMORY_WHOLE_ALLOCATION_ARM: cl_import_properties_arm =
+    cl_import_properties_arm::MAX;
 
 pub const CL_DEVICE_SVM_CAPABILITIES_ARM: cl_device_info = 0x40B6;
 
@@ -457,7 +590,10 @@ pub const CL_COMMAND_TERMINATION_CONTROLLED_SUCCESS_ARM: cl_command_termination_
 pub const CL_COMMAND_TERMINATION_CONTROLLED_FAILURE_ARM: cl_command_termination_reason_arm = 2;
 pub const CL_COMMAND_TERMINATION_ERROR_ARM: cl_command_termination_reason_arm = 3;
 
-// cl_intel_thread_local_exec extension
+// cl_arm_protected_memory_allocation
+pub const CL_MEM_PROTECTED_ALLOC_ARM: cl_bitfield = 1 << 36;
+
+// cl_intel_exec_by_local_thread extension
 pub const CL_QUEUE_THREAD_LOCAL_EXEC_ENABLE_INTEL: cl_bitfield = 1 << 31;
 
 // cl_intel_device_attribute_query
@@ -754,13 +890,19 @@ pub const CL_AVC_ME_INTERLACED_SCAN_TOP_FIELD_INTEL: cl_intel_avc_motion_estimat
 pub const CL_AVC_ME_INTERLACED_SCAN_BOTTOM_FIELD_INTEL: cl_intel_avc_motion_estimation = 0x1;
 
 // cl_intel_unified_shared_memory extension
+pub type cl_device_unified_shared_memory_capabilities_intel = cl_bitfield;
+pub type cl_mem_properties_intel = cl_properties;
+pub type cl_mem_alloc_flags_intel = cl_bitfield;
+pub type cl_mem_info_intel = cl_uint;
+pub type cl_unified_shared_memory_type_intel = cl_uint;
+pub type cl_mem_advice_intel = cl_uint;
+
 pub const CL_DEVICE_HOST_MEM_CAPABILITIES_INTEL: cl_device_info = 0x4190;
 pub const CL_DEVICE_DEVICE_MEM_CAPABILITIES_INTEL: cl_device_info = 0x4191;
 pub const CL_DEVICE_SINGLE_DEVICE_SHARED_MEM_CAPABILITIES_INTEL: cl_device_info = 0x4192;
 pub const CL_DEVICE_CROSS_DEVICE_SHARED_MEM_CAPABILITIES_INTEL: cl_device_info = 0x4193;
 pub const CL_DEVICE_SHARED_SYSTEM_MEM_CAPABILITIES_INTEL: cl_device_info = 0x4194;
 
-pub type cl_device_unified_shared_memory_capabilities_intel = cl_bitfield;
 pub const CL_UNIFIED_SHARED_MEMORY_ACCESS_INTEL:
     cl_device_unified_shared_memory_capabilities_intel = 1 << 0;
 pub const CL_UNIFIED_SHARED_MEMORY_ATOMIC_ACCESS_INTEL:
@@ -770,26 +912,22 @@ pub const CL_UNIFIED_SHARED_MEMORY_CONCURRENT_ACCESS_INTEL:
 pub const CL_UNIFIED_SHARED_MEMORY_CONCURRENT_ATOMIC_ACCESS_INTEL:
     cl_device_unified_shared_memory_capabilities_intel = 1 << 3;
 
-pub type cl_mem_properties_intel = cl_properties;
 pub const CL_MEM_ALLOC_FLAGS_INTEL: cl_mem_properties_intel = 0x4195;
 
-pub type cl_mem_alloc_flags_intel = cl_bitfield;
 pub const CL_MEM_ALLOC_WRITE_COMBINED_INTEL: cl_mem_alloc_flags_intel = 1 << 0;
+pub const CL_MEM_ALLOC_INITIAL_PLACEMENT_DEVICE_INTEL: cl_mem_alloc_flags_intel = 1 << 1;
+pub const CL_MEM_ALLOC_INITIAL_PLACEMENT_HOST_INTEL: cl_mem_alloc_flags_intel = 1 << 2;
 
-pub type cl_mem_info_intel = cl_uint;
 pub const CL_MEM_ALLOC_TYPE_INTEL: cl_mem_info_intel = 0x419A;
 pub const CL_MEM_ALLOC_BASE_PTR_INTEL: cl_mem_info_intel = 0x419B;
 pub const CL_MEM_ALLOC_SIZE_INTEL: cl_mem_info_intel = 0x419C;
 pub const CL_MEM_ALLOC_DEVICE_INTEL: cl_mem_info_intel = 0x419D;
 
-pub type cl_unified_shared_memory_type_intel = cl_uint;
 pub const CL_MEM_TYPE_UNKNOWN_INTEL: cl_unified_shared_memory_type_intel = 0x4196;
 pub const CL_MEM_TYPE_HOST_INTEL: cl_unified_shared_memory_type_intel = 0x4197;
 pub const CL_MEM_TYPE_DEVICE_INTEL: cl_unified_shared_memory_type_intel = 0x4198;
 pub const CL_MEM_TYPE_SHARED_INTEL: cl_unified_shared_memory_type_intel = 0x4199;
 
-pub type cl_mem_advice_intel = cl_uint;
-
 pub const CL_KERNEL_EXEC_INFO_INDIRECT_HOST_ACCESS_INTEL: cl_kernel_exec_info = 0x4200;
 pub const CL_KERNEL_EXEC_INFO_INDIRECT_DEVICE_ACCESS_INTEL: cl_kernel_exec_info = 0x4201;
 pub const CL_KERNEL_EXEC_INFO_INDIRECT_SHARED_ACCESS_INTEL: cl_kernel_exec_info = 0x4202;
@@ -856,6 +994,161 @@ pub const CL_QUEUE_CAPABILITY_KERNEL_INTEL: cl_command_queue_capabilities_intel
 #[cfg_attr(not(target_os = "macos"), link(name = "OpenCL"))]
 #[cfg_attr(target_os = "macos", link(name = "OpenCL", kind = "framework"))]
 extern "system" {
+    pub fn clCreateCommandBufferKHR(
+        num_queues: cl_uint,
+        queues: *const cl_command_queue,
+        properties: *const cl_command_buffer_properties_khr,
+        errcode_ret: *mut cl_int,
+    ) -> cl_command_buffer_khr;
+
+    pub fn clFinalizeCommandBufferKHR(command_buffer: cl_command_buffer_khr) -> cl_int;
+
+    pub fn clRetainCommandBufferKHR(command_buffer: cl_command_buffer_khr) -> cl_int;
+
+    pub fn clReleaseCommandBufferKHR(command_buffer: cl_command_buffer_khr) -> cl_int;
+
+    pub fn clEnqueueCommandBufferKHR(
+        num_queues: cl_uint,
+        queues: *mut cl_command_queue,
+        command_buffer: cl_command_buffer_khr,
+        num_events_in_wait_list: cl_uint,
+        event_wait_list: *const cl_event,
+        event: *mut cl_event,
+    ) -> cl_int;
+
+    pub fn clCommandBarrierWithWaitListKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandCopyBufferKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        src_buffer: cl_mem,
+        dst_buffer: cl_mem,
+        src_offset: size_t,
+        dst_offset: size_t,
+        size: size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandCopyBufferRectKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        src_buffer: cl_mem,
+        dst_buffer: cl_mem,
+        src_origin: *const size_t,
+        dst_origin: *const size_t,
+        region: *const size_t,
+        src_row_pitch: size_t,
+        src_slice_pitch: size_t,
+        dst_row_pitch: size_t,
+        dst_slice_pitch: size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandCopyBufferToImageKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        src_buffer: cl_mem,
+        dst_image: cl_mem,
+        src_offset: size_t,
+        dst_origin: *const size_t,
+        region: *const size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandCopyImageKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        src_image: cl_mem,
+        dst_image: cl_mem,
+        src_origin: *const size_t,
+        dst_origin: *const size_t,
+        region: *const size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandCopyImageToBufferKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        src_image: cl_mem,
+        dst_buffer: cl_mem,
+        src_origin: *const size_t,
+        region: *const size_t,
+        dst_offset: size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandFillBufferKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        buffer: cl_mem,
+        pattern: *const c_void,
+        pattern_size: size_t,
+        offset: size_t,
+        size: size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandFillImageKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        image: cl_mem,
+        fill_color: *const c_void,
+        origin: *const size_t,
+        region: *const size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clCommandNDRangeKernelKHR(
+        command_buffer: cl_command_buffer_khr,
+        command_queue: cl_command_queue,
+        properties: *const cl_ndrange_kernel_command_properties_khr,
+        kernel: cl_kernel,
+        work_dim: cl_uint,
+        global_work_offset: *const size_t,
+        global_work_size: *const size_t,
+        local_work_size: *const size_t,
+        num_sync_points_in_wait_list: cl_uint,
+        sync_point_wait_list: *const cl_sync_point_khr,
+        sync_point: *mut cl_sync_point_khr,
+        mutable_handle: *mut cl_mutable_command_khr,
+    ) -> cl_int;
+
+    pub fn clGetCommandBufferInfoKHR(
+        command_buffer: cl_command_buffer_khr,
+        param_name: cl_command_buffer_info_khr,
+        param_value_size: size_t,
+        param_value: *mut c_void,
+        param_value_size_ret: *mut size_t,
+    ) -> cl_int;
+
     pub fn clSetMemObjectDestructorAPPLE(
         memobj: cl_mem,
         pfn_notify: extern "C" fn(cl_mem, *const c_void),
@@ -988,6 +1281,71 @@ extern "system" {
         suggested_local_work_size: *mut size_t,
     ) -> cl_int;
 
+    pub fn clEnqueueAcquireExternalMemObjectsKHR(
+        command_queue: cl_command_queue,
+        num_mem_objects: cl_uint,
+        mem_objects: *const cl_mem,
+        num_events_in_wait_list: cl_uint,
+        event_wait_list: *const cl_event,
+        event: *mut cl_event,
+    ) -> cl_int;
+
+    pub fn clEnqueueReleaseExternalMemObjectsKHR(
+        command_queue: cl_command_queue,
+        num_mem_objects: cl_uint,
+        mem_objects: *const cl_mem,
+        num_events_in_wait_list: cl_uint,
+        event_wait_list: *const cl_event,
+        event: *mut cl_event,
+    ) -> cl_int;
+
+    pub fn clGetSemaphoreHandleForTypeKHR(
+        sema_object: cl_semaphore_khr,
+        device: cl_device_id,
+        handle_type: cl_external_semaphore_handle_type_khr,
+        handle_size: size_t,
+        handle_ptr: *mut c_void,
+        handle_size_ret: *mut size_t,
+    ) -> cl_int;
+
+    pub fn clCreateSemaphoreWithPropertiesKHR(
+        context: cl_context,
+        sema_props: *const cl_semaphore_properties_khr,
+        errcode_ret: *mut cl_int,
+    ) -> cl_semaphore_khr;
+
+    pub fn clEnqueueWaitSemaphoresKHR(
+        command_queue: cl_command_queue,
+        num_sema_objects: cl_uint,
+        sema_objects: *const cl_semaphore_khr,
+        sema_payload_list: *const cl_semaphore_payload_khr,
+        num_events_in_wait_list: cl_uint,
+        event_wait_list: *const cl_event,
+        event: *mut cl_event,
+    ) -> cl_int;
+
+    pub fn clEnqueueSignalSemaphoresKHR(
+        command_queue: cl_command_queue,
+        num_sema_objects: cl_uint,
+        sema_objects: *const cl_semaphore_khr,
+        sema_payload_list: *const cl_semaphore_payload_khr,
+        num_events_in_wait_list: cl_uint,
+        event_wait_list: *const cl_event,
+        event: *mut cl_event,
+    ) -> cl_int;
+
+    pub fn clGetSemaphoreInfoKHR(
+        sema_object: cl_semaphore_khr,
+        param_name: cl_semaphore_info_khr,
+        param_value_size: size_t,
+        param_value: *mut c_void,
+        param_value_size_ret: *mut size_t,
+    ) -> cl_int;
+
+    pub fn clReleaseSemaphoreKHR(sema_object: cl_semaphore_khr) -> cl_int;
+
+    pub fn clRetainSemaphoreKHR(sema_object: cl_semaphore_khr) -> cl_int;
+
     pub fn clImportMemoryARM(
         context: cl_context,
         flags: cl_mem_flags,
@@ -1124,7 +1482,7 @@ extern "system" {
         errcode_ret: *mut cl_int,
     );
 
-    pub fn clMemFreeINTEL(context: cl_context) -> cl_int;
+    pub fn clMemFreeINTEL(context: cl_context, ptr: *mut c_void) -> cl_int;
 
     pub fn clMemBlockingFreeINTEL(context: cl_context, ptr: *mut c_void) -> cl_int;
 
@@ -1143,16 +1501,6 @@ extern "system" {
         arg_value: *const c_void,
     ) -> cl_int;
 
-    pub fn clEnqueueMemsetINTEL(
-        command_queue: cl_command_queue,
-        dst_ptr: *mut c_void,
-        value: cl_int,
-        size: size_t,
-        num_events_in_wait_list: cl_uint,
-        event_wait_list: *const cl_event,
-        event: *mut cl_event,
-    ) -> cl_int;
-
     pub fn clEnqueueMemFillINTEL(
         command_queue: cl_command_queue,
         dst_ptr: *mut c_void,
@@ -1175,6 +1523,16 @@ extern "system" {
         event: *mut cl_event,
     ) -> cl_int;
 
+    pub fn clEnqueueMemAdviseINTEL(
+        command_queue: cl_command_queue,
+        ptr: *const c_void,
+        size: size_t,
+        advice: cl_mem_advice_intel,
+        num_events_in_wait_list: cl_uint,
+        event_wait_list: *const cl_event,
+        event: *mut cl_event,
+    ) -> cl_int;
+
     pub fn clEnqueueMigrateMemINTEL(
         command_queue: cl_command_queue,
         ptr: *const c_void,
@@ -1185,11 +1543,11 @@ extern "system" {
         event: *mut cl_event,
     ) -> cl_int;
 
-    pub fn clEnqueueMemAdviseINTEL(
+    pub fn clEnqueueMemsetINTEL(
         command_queue: cl_command_queue,
-        ptr: *const c_void,
+        dst_ptr: *mut c_void,
+        value: cl_int,
         size: size_t,
-        advice: cl_mem_advice_intel,
         num_events_in_wait_list: cl_uint,
         event_wait_list: *const cl_event,
         event: *mut cl_event,
