@@ -35,6 +35,7 @@ pub enum InfoType {
     Luid([u8; CL_LUID_SIZE_KHR]),
     Uuid([u8; CL_UUID_SIZE_KHR]),
     VecUchar(Vec<u8>),
+    VecUshort(Vec<u32>),
     VecUlong(Vec<u64>),
     VecSize(Vec<usize>),
     VecIntPtr(Vec<isize>),
@@ -101,6 +102,12 @@ impl From<InfoType> for Vec<u8> {
     }
 }
 
+impl From<InfoType> for Vec<u32> {
+    fn from(value: InfoType) -> Self {
+        match_info_type!(value, InfoType::VecUshort)
+    }
+}
+
 impl From<InfoType> for Vec<u64> {
     fn from(value: InfoType) -> Self {
         match_info_type!(value, InfoType::VecUlong)
